@@ -1,6 +1,8 @@
 
-from generallibrary import CodeLine, Markdown, Date, exclusive, deco_cache, cache_clear, Timer
+import json
+
 from generalfile import Path
+from generallibrary import CodeLine, Markdown, Date, deco_cache, Timer, Log
 
 
 class GenerateFile:
@@ -12,8 +14,10 @@ class GenerateFile:
         self.aesthetic = aesthetic
         self.overwrite = overwrite
 
+        Log().debug(f"Creating {text_func.__name__} handler for {packager.name} with path {packager.path}")
+
         self.relative_path = path.relative(base=packager.path)
-        self.path = packager.path / self.relative_path
+        self.path = packager.path / self.relative_path  # HERE ** This doesn't work if Packager's path is None
 
     def generate(self):
         """ Generate actual file. """
@@ -32,32 +36,69 @@ class _PackagerFiles:
 
     _todo_header = "Todo"
 
-    def __init_post__(self):
+    @property
+    @deco_cache()
+    def files(self):
         """ Todo: Watermark generated files to prevent mistake of thinking you can modify them directly.
-
             :param generalpackager.Packager self: """
-        self.file_setup =           GenerateFile(self.localrepo.get_setup_path(), self.generate_setup, self, aesthetic=False)
-        self.file_manifest =        GenerateFile(self.localrepo.get_manifest_path(), self.generate_manifest, self, aesthetic=False)
+        files = [
+            GenerateFile(self.localrepo.get_git_exclude_path(), self.generate_git_exclude, self, aesthetic=True),
+            GenerateFile(self.localrepo.get_license_path(), self.generate_license, self, aesthetic=True),
+            GenerateFile(self.localrepo.get_workflow_path(), self.generate_workflow, self, aesthetic=True),
+            GenerateFile(self.localrepo.get_readme_path(), self.generate_readme, self, aesthetic=True),
+            GenerateFile(self.localrepo.get_generate_path(), self.generate_generate, self, aesthetic=True),
+        ]
 
-        self.file_git_exclude =     GenerateFile(self.localrepo.get_git_exclude_path(), self.generate_git_exclude, self, aesthetic=True)
-        self.file_license =         GenerateFile(self.localrepo.get_license_path(), self.generate_license, self, aesthetic=True)
-        self.file_workflow =        GenerateFile(self.localrepo.get_workflow_path(), self.generate_workflow, self, aesthetic=True)
-        self.file_readme =          GenerateFile(self.localrepo.get_readme_path(), self.generate_readme, self, aesthetic=True)
-        self.file_generate =        GenerateFile(self.localrepo.get_generate_path(), self.generate_generate, self, aesthetic=True)
+        if self.is_python():
+            files.extend([
+                GenerateFile(self.localrepo.get_setup_path(), self.generate_setup, self, aesthetic=False),
+                GenerateFile(self.localrepo.get_manifest_path(), self.generate_manifest, self, aesthetic=False),
+                GenerateFile(self.localrepo.get_init_path(), self.generate_init, self, aesthetic=False, overwrite=False),
+                GenerateFile(self.localrepo.get_randomtesting_path(), self.generate_randomtesting, self, aesthetic=True, overwrite=False),
+                GenerateFile(self.localrepo.get_test_template_path(), self.generate_test_python, self, aesthetic=False, overwrite=False),
+            ])
 
-        self.file_init =            GenerateFile(self.localrepo.get_init_path(), self.generate_init, self, aesthetic=False, overwrite=False)
-        self.file_randomtesting =   GenerateFile(self.localrepo.get_randomtesting_path(), self.generate_randomtesting, self, aesthetic=True, overwrite=False)
-        self.file_test_template =   GenerateFile(self.localrepo.get_test_template_path(), self.generate_test_template, self, aesthetic=False, overwrite=False)
+        elif self.is_node():
+            files.extend([
+                GenerateFile(self.localrepo.get_npm_ignore_path(), self.generate_npm_ignore, self, aesthetic=True),
+                GenerateFile(self.localrepo.get_index_js_path(), self.generate_index_js, self, aesthetic=False, overwrite=False),
+                GenerateFile(self.localrepo.get_test_js_path(), self.generate_test_node, self, aesthetic=False, overwrite=False),
+                GenerateFile(self.localrepo.get_package_json_path(), self.generate_package_json, self, aesthetic=False),
+            ])
 
-        self.files = [getattr(self, key) for key in dir(self) if key.startswith("file_")]  # type: list[GenerateFile]
-        self.files_by_relative_path = {file.relative_path: file for file in self.files}
+        return files
 
+    @deco_cache()
+    def all_files_by_relative_path(self):
+        """ :param generalpackager.Packager self: """
+        return {file.relative_path: file for file in self.files}
+
+    def file_by_relative_path(self, path):
+        """ :param generalpackager.Packager self:
+            :param Path or str path:
+            :rtype: GenerateFile """
+        path = Path(path)
+        try:
+            return self.all_files_by_relative_path()[path.relative(base=self.path)]
+        except KeyError:
+            return None
+
+    @property
+    @deco_cache()
+    def file_secret_readme(self):
+        """ :param generalpackager.Packager self: """
         # Organization secret name is .github, user secret name is user
-        secret_readme_path = self.localrepo.get_org_readme_path() if self.name == ".github" else self.localrepo.get_readme_path()
-        self.file_secret_readme = GenerateFile(secret_readme_path, self.generate_personal_readme, self, aesthetic=True)
+        if self.name == ".github":
+            secret_readme_path = self.localrepo.get_org_readme_path()
+        else:
+            secret_readme_path = self.localrepo.get_readme_path()
+
+        return GenerateFile(secret_readme_path, self.generate_personal_readme, self, aesthetic=True)
 
     def get_new_packager(self):
-        """ :param generalpackager.Packager self: """
+        """ Todo: Generalize get_new_packager which calls recycle_clear on all attributes.
+
+            :param generalpackager.Packager self: """
         self.recycle_clear()
         self.localrepo.recycle_clear()
         self.localmodule.recycle_clear()
@@ -65,17 +106,24 @@ class _PackagerFiles:
         self.pypi.recycle_clear()
         return type(self)(self.name)
 
-    def create_blank_locally(self, install=True):
+    @classmethod
+    def create_blank_locally_python(cls, path, install=True):
         """ Create a new general package locally only.
 
-            :param generalpackager.Packager self:
+            :param generalpackager.Packager or Any cls:
+            :param Path or str path:
             :param install: Whether to pip install. """
-        self.localrepo.write_metadata()
-        self.generate_localfiles()
-        if install:
-            self.localrepo.pip_install()
+        path = Path(path)
+        assert path.empty()
+        packager = cls(name=path.name(), path=path, target=cls.Targets.python)
 
-        new_self = self.get_new_packager()  # Reset caches to get updated files
+        packager.localrepo.metadata.write_config()
+        packager.generate_localfiles()
+
+        if install:
+            packager.localrepo.pip_install_editable()
+
+        new_self = packager.get_new_packager()  # Reset caches to get updated files
         new_self.generate_localfiles()
 
     def relative_path_is_aesthetic(self, relative_path):
@@ -84,7 +132,7 @@ class _PackagerFiles:
             :param generalpackager.Packager self:
             :param Path or str relative_path: """
         relative_path = Path(relative_path).relative(self.path)
-        aesthetic_attr = getattr(self.files_by_relative_path.get(relative_path, None), "aesthetic", None)
+        aesthetic_attr = getattr(self.all_files_by_relative_path().get(relative_path, None), "aesthetic", None)
         if aesthetic_attr is None:
             if relative_path.match(*self.extra_aesthetic):
                 return True
@@ -122,6 +170,7 @@ class _PackagerFiles:
             :param aesthetic: """
         return self._compare_local(platform=self.github, aesthetic=aesthetic)
 
+
     def compare_local_to_pypi(self, aesthetic=None):
         """ Get a list of changed files compared to pypi with optional aesthetic files.
 
@@ -140,16 +189,16 @@ class _PackagerFiles:
             "name": f'"{self.localrepo.name}"',
             "author": f"'{self.author}'",
             "author_email": f'"{self.email}"',
-            "version": f'"{self.localrepo.version}"',
-            "description": f'"{self.localrepo.description}"',
+            "version": f'"{self.localrepo.metadata.version}"',
+            "description": f'"{self.localrepo.metadata.description}"',
             "long_description": "long_description",
             "long_description_content_type": '"text/markdown"',
-            "install_requires": self.localrepo.install_requires,
+            "install_requires": self.localrepo.metadata.install_requires,
             "url": f'"{self.github.url}"',
             "license": f'"{self.license}"',
             "python_requires": f'">={self.python[0]}, <{last_version_bumped_micro}"',
             "packages": 'find_namespace_packages(exclude=("build*", "dist*"))',
-            "extras_require": self.localrepo.extras_require,
+            "extras_require": self.localrepo.metadata.extras_require,
             "classifiers": self.get_classifiers(),
             # "include_package_data": True,
         }
@@ -187,7 +236,7 @@ class _PackagerFiles:
         default_manifest = [
             self.localrepo.get_metadata_path().relative(self.path),
         ]
-        return "\n".join([f"include {path}" for path in self.localrepo.manifest + default_manifest])
+        return "\n".join([f"include {path}" for path in self.localrepo.metadata.manifest + default_manifest])
 
     def generate_git_exclude(self):
         """ Generate git exclude file.
@@ -199,7 +248,9 @@ class _PackagerFiles:
         """ Generate LICENSE by using Packager.license.
 
             :param generalpackager.Packager self: """
-        text = Path(self.localrepo.get_repos_path() / f"generalpackager/generalpackager/licenses/{self.license}").text.read()
+        # text = Path(self.localrepo.get_repos_path() / f"generalpackager/generalpackager/licenses/{self.license}").text.read()
+        text = (type(self)("generalpackager").path / "generalpackager/licenses" / self.license).text.read()
+
         assert "$" in text
         text = text.replace("$year", str(Date.now().datetime.year))
         text = text.replace("$author", self.author)
@@ -214,12 +265,13 @@ class _PackagerFiles:
         workflow = CodeLine()
         workflow.indent_str = " " * 2
 
-        workflow.add_node("name: workflow")
-        workflow.add_node(self.get_triggers())
+        workflow.add_node(self._get_name())
+        workflow.add_node(self._get_triggers())
+        workflow.add_node(self._get_defaults())
 
         jobs = workflow.add_node("jobs:")
-        jobs.add_node(self.get_unittest_job())
-        jobs.add_node(self.get_sync_job())
+        jobs.add_node(self._get_unittest_job())
+        jobs.add_node(self._get_sync_job())
 
         return workflow.text()
 
@@ -257,7 +309,9 @@ class _PackagerFiles:
         """ Generate personal readme markdown and overwrite README.md in local repo.
 
             :param generalpackager.Packager self: """
-        ordered_packagers = type(self)().get_ordered_packagers()
+        # ordered_packagers = type(self)().get_ordered_packagers()
+        ordered_packagers = self.get_ordered_packagers(include_private=False)
+
 
         # Description
         markdown = Markdown(header="ManderaGeneral").add_list_lines(
@@ -303,7 +357,7 @@ class _PackagerFiles:
 
         return top
 
-    def generate_test_template(self):
+    def generate_test_python(self):
         """ Generate test template.
 
             :param generalpackager.Packager self: """
@@ -313,6 +367,57 @@ class _PackagerFiles:
 
         return top
 
+    def generate_npm_ignore(self):
+        """ Generate .npmignore
+
+            :param generalpackager.Packager self: """
+        return "\n".join(self.npm_ignore_lines)
+
+    def generate_index_js(self):
+        """ Generate index.js
+
+            :param generalpackager.Packager self: """
+        top = CodeLine()
+        top.add_node(CodeLine('exports.Vec2 = require("./vec2");', space_before=1, space_after=1))
+
+        return top
+
+    def generate_test_node(self):
+        """ Generate test template.
+
+            :param generalpackager.Packager self: """
+        top = CodeLine()
+        top.add_node("/**")
+        top.add_node(" * @jest-environment jsdom")
+        top.add_node(CodeLine(" */", space_after=1))
+        top.add_node(CodeLine("// https://jestjs.io/docs/configuration#testenvironment-string", space_after=1))
+        top.add_node(CodeLine('const Vec2 = require("./vec2");', space_after=1))
+        top.add_node('test("Vec2 initializing", () => {').add_node("expect(new Vec2().x).toBe(0);")
+        top.add_node("})")
+
+        return top
+
+    def generate_package_json(self):
+        """ Generate test template.
+
+            :param generalpackager.Packager self: """
+        info = {
+            "name": self.localrepo.name,
+            "version": str(self.localrepo.metadata.version),
+            "description": self.localrepo.metadata.description,
+            "scripts": {
+                "start": "parcel index.html",
+                "build": "parcel build index.html",
+                "test": "jest"
+            },
+            "dependencies": {dep: "latest" for dep in self.localrepo.dependencies},
+            "devDependencies": {dep: "latest" for dep in self.localrepo.devDependencies},
+            "keywords": self.get_topics(),
+            "license": self.license,
+            "author": self.author,
+        }
+        return json.dumps(info, indent=4)
+
     def generate_localfiles(self, aesthetic=True, print_out=False):
         """ Generate all local files.
 
@@ -322,7 +427,7 @@ class _PackagerFiles:
         timer = Timer()
 
         # Not in files because it writes with json not text, it's also a bit unique
-        self.localrepo.write_metadata()
+        self.localrepo.metadata.write_config()
 
         for generate in self.files:
             if aesthetic or not generate.aesthetic:
