@@ -1,12 +1,11 @@
+from generallibrary import Log
 
-from generalpackager.api.local_module import LocalModule
+from generalpackager.api.localmodule import LocalModule
 
 import unittest
 
 
 class TestLocalModule(unittest.TestCase):
-    """ Skipped tests:
-    """
     def test_exists(self):
         self.assertEqual(True, LocalModule("generalpackager").exists())
         self.assertEqual(False, LocalModule("doesntexist").exists())
